@@ -15,6 +15,7 @@
 #include <smooth/io/drivers/driver_ansi.h>
 #include <smooth/io/drivers/driver_memory.h>
 
+#include <smooth/files/directory.h>
 #include <smooth/files/file.h>
 
 #include <stdio.h>
@@ -50,6 +51,7 @@ S::IO::OutStream::OutStream(Int type, const String &fileName, Int mode)
 	inStream	= NIL;
 
 	if (type != STREAM_FILE)		   { lastError = IO_ERROR_BADPARAM;			return; }
+	if (Directory(fileName).Exists())	   { lastError = IO_ERROR_BADPARAM;			return; }
 
 	driver		= new DriverANSI(File(fileName), mode);
 
